@@ -2,6 +2,7 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use crate::either::EitherCart;
 #[cfg(feature = "alloc")]
 use crate::erased::{ErasedBoxCart, ErasedRcCart};
 use crate::trait_hack::YokeTraitHack;
@@ -21,12 +22,13 @@ use alloc::sync::Arc;
 /// A Cow-like borrowed object "yoked" to its backing data.
 ///
 /// This allows things like zero copy deserialized data to carry around
-/// shared references to their backing buffer.
+/// shared references to their backing buffer, by "erasing" their static lifetime
+/// and turning it into a dynamically managed one.
 ///
 /// `Y` (the [`Yokeable`]) is the object containing the references,
 /// and will typically be of the form `Foo<'static>`. The `'static` is
-/// not the actual lifetime of the data, rather it is a convenient way to erase
-/// the lifetime and make it dynamic.
+/// not the actual lifetime of the data, rather it is a convenient way to mark the
+/// erased lifetime and make it dynamic.
 ///
 /// `C` is the "cart", which `Y` may contain references to. After the yoke is constructed,
 /// the cart serves little purpose except to guarantee that `Y`'s references remain valid
@@ -35,6 +37,17 @@ use alloc::sync::Arc;
 /// The primary constructor for [`Yoke`] is [`Yoke::attach_to_cart()`]. Several variants of that
 /// constructor are provided to serve numerous types of call sites and `Yoke` signatures.
 ///
+/// The key behind this type is [`Yoke::get()`], where calling [`.get()`][Yoke::get] on a type like
+/// `Yoke<Cow<'static, str>, _>` will get you a short-lived `&'a Cow<'a, str>`, restricted to the
+/// lifetime of the borrow used during `.get()`. This is entirely safe since the `Cow` borrows from
+/// the cart type `C`, which cannot be interfered with as long as the `Yoke` is borrowed by `.get
+/// ()`. `.get()` protects access by essentially reifying the erased lifetime to a safe local one
+/// when necessary.
+///
+/// Furthermore, there are various [`.project()`][Yoke::project] methods that allow turning a `Yoke`
+/// into another `Yoke` containing a different type that may contain elements of the original yoked
+/// value. See the [`Yoke::project()`] docs for more details.
+///
 /// In general, `C` is a concrete type, but it is also possible for it to be a trait object;
 /// for more information, see [`IsCovariant`].
 ///
@@ -76,9 +89,35 @@ impl<Y: for<'a> Yokeable<'a>, C: StableDeref> Yoke<Y, C> {
     ///
     /// See also [`Yoke::try_attach_to_cart()`] to return a `Result` from the closure.
     ///
-    /// Due to [compiler bug #84937](https://github.com/rust-lang/rust/issues/84937), call sites
-    /// for this function may not compile; if this happens, use
+    /// Call sites for this function may not compile; if this happens, use
     /// [`Yoke::attach_to_cart_badly()`] instead.
+    ///
+    /// # Examples
+    ///
+    /// The following code does not currently compile.
+    /// See [#1061](https://github.com/unicode-org/icu4x/issues/1061).
+    ///
+    /// ```compile_fail
+    /// # use yoke::{Yoke, Yokeable};
+    /// # use std::rc::Rc;
+    /// # use std::borrow::Cow;
+    /// # fn load_from_cache(_filename: &str) -> Rc<[u8]> {
+    /// #     // dummy implementation
+    /// #     Rc::new([0x5, 0, 0, 0, 0, 0, 0, 0, 0x68, 0x65, 0x6c, 0x6c, 0x6f])
+    /// # }
+    ///
+    /// fn load_object(filename: &str) -> Yoke<Cow<'static, str>, Rc<[u8]>> {
+    ///     let rc: Rc<[u8]> = load_from_cache(filename);
+    ///     Yoke::<Cow<'static, str>, Rc<[u8]>>::attach_to_cart(rc, |data: &[u8]| {
+    ///         // essentially forcing a #[serde(borrow)]
+    ///         Cow::Borrowed(bincode::deserialize(data).unwrap())
+    ///     })
+    /// }
+    ///
+    /// let yoke: Yoke<Cow<str>, _> = load_object("filename.bincode");
+    /// assert_eq!(&**yoke.get(), "hello");
+    /// assert!(matches!(yoke.get(), &Cow::Borrowed(_)));
+    /// ```
     pub fn attach_to_cart<F>(cart: C, f: F) -> Self
     where
         F: for<'de> FnOnce(&'de <C as Deref>::Target) -> <Y as Yokeable<'de>>::Output,
@@ -93,9 +132,9 @@ impl<Y: for<'a> Yokeable<'a>, C: StableDeref> Yoke<Y, C> {
     /// Construct a [`Yoke`] by yokeing an object to a cart. If an error occurs in the
     /// deserializer function, the error is passed up to the caller.
     ///
-    /// Due to [compiler bug #84937](https://github.com/rust-lang/rust/issues/84937), call sites
-    /// for this function may not compile; if this happens, use
+    /// Call sites for this function may not compile; if this happens, use
     /// [`Yoke::try_attach_to_cart_badly()`] instead.
+    /// See [#1061](https://github.com/unicode-org/icu4x/issues/1061).
     pub fn try_attach_to_cart<E, F>(cart: C, f: F) -> Result<Self, E>
     where
         F: for<'de> FnOnce(&'de <C as Deref>::Target) -> Result<<Y as Yokeable<'de>>::Output, E>,
@@ -286,9 +325,9 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     ///
     /// - `f()` must not panic
     /// - References from the yokeable `Y` should still be valid for the lifetime of the
-    ///   returned cart type.
+    ///   returned cart type `C`.
     ///
-    /// Typically, this means implementing `f` as something which _wraps_ the inner cart type.
+    /// Typically, this means implementing `f` as something which _wraps_ the inner cart type `C`.
     /// `Yoke` only really cares about destructors for its carts so it's fine to erase other
     /// information about the cart, as long as the backing data will still be destroyed at the
     /// same time.
@@ -390,7 +429,7 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
         self.yokeable.transform_mut(f)
     }
 
-    /// Helper function allowing one to wrap the cart type in an `Option<T>`.
+    /// Helper function allowing one to wrap the cart type `C` in an `Option<T>`.
     #[inline]
     pub fn wrap_cart_in_option(self) -> Yoke<Y, Option<C>> {
         unsafe {
@@ -504,7 +543,7 @@ unsafe impl<T: CloneableCart> CloneableCart for Option<T> {}
 unsafe impl<'a, T: ?Sized> CloneableCart for &'a T {}
 unsafe impl CloneableCart for () {}
 
-/// Clone requires that the cart derefs to the same address after it is cloned. This works for
+/// Clone requires that the cart type `C` derefs to the same address after it is cloned. This works for
 /// Rc, Arc, and &'a T.
 ///
 /// For other cart types, clone `.backing_cart()` and re-use `.attach_to_cart()`; however, doing
@@ -572,18 +611,12 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     /// described in [#86702](https://github.com/rust-lang/rust/issues/86702). This parameter
     /// should just be ignored in the function.
     ///
-    /// Furthermore,
-    /// [compiler bug #84937](https://github.com/rust-lang/rust/issues/84937) prevents
-    /// this from taking a capturing closure, however [`Yoke::project_with_capture()`]
-    /// can be used for the same use cases.
-    ///
+    /// To capture data and pass it to the closure, use [`Yoke::project_with_capture()`].
+    /// See [#1061](https://github.com/unicode-org/icu4x/issues/1061).
     ///
     /// This can be used, for example, to transform data from one format to another:
     ///
-    /// ***[#1061](https://github.com/unicode-org/icu4x/issues/1061): The following example
-    /// requires Rust 1.56.***
-    ///
-    /// ```rust,ignore
+    /// ```
     /// # use std::rc::Rc;
     /// # use yoke::Yoke;
     /// #
@@ -595,10 +628,7 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     ///
     /// This can also be used to create a yoke for a subfield
     ///
-    /// ***[#1061](https://github.com/unicode-org/icu4x/issues/1061): The following example
-    /// requires Rust 1.56.***
-    ///
-    /// ```rust,ignore
+    /// ```
     /// # use std::borrow::Cow;
     /// # use yoke::{Yoke, Yokeable};
     /// # use std::mem;
@@ -682,9 +712,9 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
         }
     }
 
-    /// This is similar to [`Yoke::project`], however it works around it not being able to
-    /// use `FnOnce` by using an explicit capture input, until [compiler bug #84937](https://github.com/rust-lang/rust/issues/84937)
-    /// is fixed.
+    /// This is similar to [`Yoke::project`], but it works around it not being able to
+    /// use `FnOnce` by using an explicit capture input.
+    /// See [#1061](https://github.com/unicode-org/icu4x/issues/1061).
     ///
     /// See the docs of [`Yoke::project`] for how this works.
     pub fn project_with_capture<P, T>(
@@ -707,8 +737,8 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     }
 
     /// This is similar to [`Yoke::project_cloned`], however it works around it not being able to
-    /// use `FnOnce` by using an explicit capture input, until [compiler bug #84937](https://github.com/rust-lang/rust/issues/84937)
-    /// is fixed.
+    /// use `FnOnce` by using an explicit capture input.
+    /// See [#1061](https://github.com/unicode-org/icu4x/issues/1061).
     ///
     /// See the docs of [`Yoke::project_cloned`] for how this works.
     pub fn project_cloned_with_capture<'this, P, T>(
@@ -781,14 +811,14 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
 impl<Y: for<'a> Yokeable<'a>, C: 'static + Sized> Yoke<Y, Rc<C>> {
     /// Allows type-erasing the cart in a `Yoke<Y, Rc<C>>`.
     ///
-    /// The yoke only carries around a cart type for its destructor,
+    /// The yoke only carries around a cart type `C` for its destructor,
     /// since it needs to be able to guarantee that its internal references
     /// are valid for the lifetime of the Yoke. As such, the actual type of the
     /// Cart is not very useful unless you wish to extract data out of it
     /// via [`Yoke::backing_cart()`]. Erasing the cart allows for one to mix
     /// [`Yoke`]s obtained from different sources.
     ///
-    /// In case the cart type is not already `Rc<T>`, you can use
+    /// In case the cart type `C` is not already an `Rc<T>`, you can use
     /// [`Yoke::wrap_cart_in_rc()`] to wrap it.
     ///
     /// # Example
@@ -826,14 +856,14 @@ impl<Y: for<'a> Yokeable<'a>, C: 'static + Sized> Yoke<Y, Rc<C>> {
 impl<Y: for<'a> Yokeable<'a>, C: 'static + Sized> Yoke<Y, Box<C>> {
     /// Allows type-erasing the cart in a `Yoke<Y, Box<C>>`.
     ///
-    /// The yoke only carries around a cart type for its destructor,
+    /// The yoke only carries around a cart type `C` for its destructor,
     /// since it needs to be able to guarantee that its internal references
     /// are valid for the lifetime of the Yoke. As such, the actual type of the
     /// Cart is not very useful unless you wish to extract data out of it
     /// via [`Yoke::backing_cart()`]. Erasing the cart allows for one to mix
     /// [`Yoke`]s obtained from different sources.
     ///
-    /// In case the cart type is not already `Box<T>`, you can use
+    /// In case the cart type `C` is not already `Box<T>`, you can use
     /// [`Yoke::wrap_cart_in_box()`] to wrap it.
     ///
     /// # Example
@@ -869,7 +899,7 @@ impl<Y: for<'a> Yokeable<'a>, C: 'static + Sized> Yoke<Y, Box<C>> {
 
 #[cfg(feature = "alloc")]
 impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
-    /// Helper function allowing one to wrap the cart type in a `Box<T>`.
+    /// Helper function allowing one to wrap the cart type `C` in a `Box<T>`.
     /// Can be paired with [`Yoke::erase_box_cart()`]
     ///
     /// Available with the `"alloc"` feature enabled.
@@ -880,7 +910,7 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
             self.replace_cart(Box::new)
         }
     }
-    /// Helper function allowing one to wrap the cart type in an `Rc<T>`.
+    /// Helper function allowing one to wrap the cart type `C` in an `Rc<T>`.
     /// Can be paired with [`Yoke::erase_rc_cart()`], or generally used
     /// to make the [`Yoke`] cloneable.
     ///
@@ -894,6 +924,35 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     }
 }
 
+impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
+    /// Helper function allowing one to wrap the cart type `C` in an [`EitherCart`].
+    ///
+    /// This function wraps the cart into the `A` variant. To wrap it into the
+    /// `B` variant, use [`Self::wrap_cart_in_either_b()`].
+    ///
+    /// For an example, see [`EitherCart`].
+    #[inline]
+    pub fn wrap_cart_in_either_a<B>(self) -> Yoke<Y, EitherCart<C, B>> {
+        unsafe {
+            // safe because the cart is preserved, just wrapped
+            self.replace_cart(EitherCart::A)
+        }
+    }
+    /// Helper function allowing one to wrap the cart type `C` in an [`EitherCart`].
+    ///
+    /// This function wraps the cart into the `B` variant. To wrap it into the
+    /// `A` variant, use [`Self::wrap_cart_in_either_a()`].
+    ///
+    /// For an example, see [`EitherCart`].
+    #[inline]
+    pub fn wrap_cart_in_either_b<A>(self) -> Yoke<Y, EitherCart<A, C>> {
+        unsafe {
+            // safe because the cart is preserved, just wrapped
+            self.replace_cart(EitherCart::B)
+        }
+    }
+}
+
 /// Safety docs for project()
 ///
 /// (Docs are on a private const to allow the use of compile_fail doctests)
